package com.ptc.wvs.util;

import java.beans.PropertyVetoException;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;
import java.util.StringTokenizer;

import com.ptc.wvs.common.util.WVSProperties;
import com.ptc.wvs.server.util.Structure2;
import wt.content.ApplicationData;
import wt.content.ContentHelper;
import wt.content.ContentRoleType;
import wt.content.ContentServerHelper;
import wt.content.ContentItem;
import wt.epm.EPMDocument;
import wt.fc.Persistable;
import wt.pom.Transaction;
import wt.representation.Representation;
import wt.representation.Representable;
import wt.util.WTException;
import wt.filter.NavigationCriteria;
import wt.util.WTProperties;

public class AfterEDRloader3 {

	private static final String CLASS_NAME = AfterEDRloader3.class.getName();
	public static String WT_TEMP = getWtTemp();
	private static boolean VERBOSE = getVerboseProperty();

	public static String[] renameAdditionalfiles(Representable repable,
                                     Persistable object,
                                     Representation rep,
                                     NavigationCriteria docCriteria,
                                     NavigationCriteria partCriteria,
                                     int structureType)
	{
		String[] ret = {"AfterEDRloader starts: Checking if object is EPMDoc"};

		if (object instanceof EPMDocument) {
			Transaction trx = new Transaction();
			try {
				trx.start();
				EPMDocument cadDoc = (EPMDocument)object;

				if (VERBOSE) {
					System.out.println("renameAdditionalfiles() START");
					System.out.println("renameAdditionalfiles : Param cadDoc : " + cadDoc );
				}

				ApplicationData appData = null;
				Vector<?> appDatas = null;
				String fileName = null;
				String sName = null;
				String fileExt = null;
				String postfix = null;
	            InputStream is = null;
				String attributeInfo = "";
				String pList = WVSProperties.getPropertyValue("publish.afterloadermethod.propertyincludelist");

				// get the ed file Name for this representation
				String edFileName = null;
				Representation ch = (Representation)ContentHelper.service.getContents(rep);
				appDatas = ContentHelper.getContentList(ch);
				int num_of_files = appDatas.size();

				for (int i = 0; i < appDatas.size(); i++) {
					appData = (ApplicationData)appDatas.get(i);
					if (appData.getRole() != null && appData.getRole().equals(ContentRoleType.PRODUCT_VIEW_ED)) {
						edFileName = appData.getFileName();
						break;
					}
				}

				// getPropertyStringValue
				String subdirectory = getTransferDir(edFileName);
				File pvs = new File(subdirectory, edFileName);
				if (pvs == null || !pvs.exists()) {
					ret[0] = "AfterEDRloader: Return Code - Bypass";
					return ret;
				}

				File edpFile = new File(subdirectory, "wvspvp.pvp");

				if( pList != null && pList.trim().length() > 0 ) {
					StringTokenizer tok = new StringTokenizer(pList.trim(), ",");
					while(tok.hasMoreTokens()) {
						String strTok = tok.nextToken();
						if (VERBOSE) System.out.println("wvsProperty: " + strTok);

						if (strTok.equals("epmdoc_objectid")) {
							attributeInfo = attributeInfo + "_" + cadDoc;
						} else {
							attributeInfo = attributeInfo + "_" + getAttributeValue(pvs, edpFile, strTok);
						}
					}
				}
				if (VERBOSE) System.out.println("attributeInfo: " + attributeInfo);

				for (int i = 0; i < appDatas.size(); i++) {
					String sFileName = cadDoc.getCADName();
					appData = (ApplicationData)appDatas.get(i);

					if (appData.getRole() != null && appData.getRole().equals(ContentRoleType.ADDITIONAL_FILES)) {
						fileName = appData.getFileName();
						postfix = fileName.substring(fileName.lastIndexOf("_")+1);
						fileExt = fileName.substring(fileName.lastIndexOf(".")+1);

						if (fileExt.equals("zip"))
						{
							sFileName = sFileName.substring(0, sFileName.lastIndexOf(".")) + attributeInfo +  "_" + postfix;
						} else {
							sFileName = sFileName.substring(0, sFileName.lastIndexOf(".")) + attributeInfo +  "." + fileExt;
						}
					
						appData.setFileName(sFileName);

						is = ContentServerHelper.service.findContentStream(appData);
						ContentServerHelper.service.updateContent(ch, appData, is);
					}
				}
				trx.commit();
				trx = null;

			} catch (PropertyVetoException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (WTException e) {
				e.printStackTrace();
			} finally {
				if (trx != null) {
					trx.rollback();
					trx = null;
				}
	        }

		}

	  return ret;
	}


	public static String getAttributeValue(File pvs, File edpFile, String attri) {

		String ret = "error-1";
		if( pvs.canRead() ) {
			try {
				Structure2 sed2 = new Structure2();
				sed2.setReadIncludeProperties(true);
				sed2.setReadExternalEDPFiles(true);
				sed2.readED(pvs);
				sed2.readEDP(edpFile, null);

				Structure2.Comp root = sed2.getRootComp();
				ret = sed2.getPropertyStringValue(root, attri);
				
				if (VERBOSE) {
					System.out.println("root component: " + root);
					System.out.println("attribute value: " + ret);
				}

			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		return ret;
	}


    private static String getTransferDir(String edFileName) {
		String directory = WT_TEMP + File.separator + "transfer";
		String subdirectory = null;
		File transferdir = new File(directory);

		//get all the folder under a directory
		File[] fList = transferdir.listFiles();

		int c = 0;
		for (File file : fList) {
			if (file.isDirectory()) {
				String[] files = file.list();
				for (int i = 0; i < files.length; i++) {
					// get the Canonical path to the folder created by the WVS Worker Agent during transfer of published content back from a worker
					if ((files[i]).equals(edFileName)) {
						try {
							subdirectory = file.getCanonicalPath();
							if (VERBOSE) System.out.println("Folder created by the WVS Worker Agent during transfer: " + subdirectory);
							c++;
						} catch (IOException ie) {
							ie.printStackTrace();
						}
					}
				}
			}
		}
		if ( c > 1) return null;
		return subdirectory;
	}


    public static String getWtTemp() {
		String wtHome = null;
		try {
			WTProperties props = WTProperties.getLocalProperties();
			wtHome = props.getProperty("wt.temp");
		}
		catch (IOException e) {
			e.printStackTrace();
		}
        return wtHome;
    }


	// Get Verbose property
    private static boolean getVerboseProperty() {
		boolean VERBOSE = false;
		try {
			String tmp = WVSProperties.getPropertyValue(CLASS_NAME + ".verbose");
			if ( tmp!=null && tmp.equalsIgnoreCase("true") ) {
				VERBOSE = true;
			}
		} catch (Throwable e) {
			System.out.println(CLASS_NAME  +": Error reading properties "+ CLASS_NAME + ".verbose");
			e.printStackTrace();
		}

		return VERBOSE;
	}


}